@@ -1,4 +1,4 @@
-from ..order import OrderMixin
+from ..order import OrderMixin, Order
 from .entity import Entity
 from ..models.core import GameConfig, GameMetadata
 from ..models.game import GameState
@@ -52,6 +52,25 @@ class Game(OrderMixin):
         """
         return [p for p in self.players if p._uid != player._uid]
 
+    async def on_order_fail(self, order: Order) -> None:
+        """
+        Called when an order fails
+        """
+
+    async def place_order(self, order: Order) -> None:
+        """
+        Place an order
+
+        Warning:
+            This method should NOT be used with predefined
+            orders (such as `BuildFactoryOrder`, `AcquireTechOrder`, ...)
+            see `Behaviour.place_order` instead.
+
+            Only use this method with custom order directly inheriting from
+            `Order` class.
+        """
+        return await super().place_order(order)
+
     async def _update_state(self, state: GameState):
         """
         Update instance with given state
@@ -67,4 +86,8 @@ class Game(OrderMixin):
 
         Entity._remove_deads(self._players)
 
-        await self._resolve_orders()
+        resolved_orders = await self._resolve_orders()
+
+        for order in resolved_orders:
+            if not order._succeeded:
+                await self.on_order_fail(order)
